------------------------------------------------------------------
-- VehicleSort Version 0.4.1
-- Author: Dschonny
-- Project started: 8. Nov. 2014
-- last update: 29. Nov. 2014

-- changelog:
-- V0.1:
-- Main functionality implemented
--		Vehicle sorting
--		Fast switching

-- V0.2:
-- added customizable keys (via modDesc)
-- added mouse-controls
-- fixed init to solve No-sorting-on-map-restart-bug
-- fixed P:false-output in log

-- V0.3:
-- fixed bug when buying/selling vehicles
-- fixed bug when showing multiple columns 

-- V0.4:
-- added parking functionality
-- activated multiplayer-support

-- V0.4.1
-- parking active vehicle implemented
-- fixed seldom fastswitch-bug (hopefully)

-- This mod may be distributed freely as long as it is unchanged
-----------------------------------------------------------------

vsClass ={};

-- Text Layout
vsClass.tPos = {};
vsClass.tPos.x = 0.3;								-- x Position of Textfield
vsClass.tPos.y = 0.9;								-- y Position of Textfield
vsClass.tPos.size = 0.018;							-- TextSize
vsClass.tPos.spacing = 0.005;						-- Spacing between lines
vsClass.tPos.columnWidth = 0.35;					-- Space till second column (only if needed)
vsClass.tPos.alignment = RenderText.ALIGN_LEFT;		-- Text Alignment

-- Text Colors
vsClass.tColor = {};
vsClass.tColor.standard = {1.0, 1.0, 1.0, 1.0}; -- white
vsClass.tColor.selected = {0.0, 1.0, 0.0, 1.0}; -- green
vsClass.tColor.locked = {1.0, 0.0, 0.0, 1.0}; 	-- red
vsClass.tColor.parked = {0.5, 0.5, 0.5, 0.7};	-- grey 

-- Internal Vars
vsClass.showSteerables = false;
vsClass.selectedIndex = 1;
vsClass.selectedLock = false;
vsClass.borderToTop = 1 - vsClass.tPos.y;

vsClass.vInitOrder = {};
vsClass.vUserOrder = {};

vsClass.userPath = "";
vsClass.vsBasePath = "";
vsClass.vsSavePath = "";
vsClass.vsXML = "";
vsClass.saveFile = 0;

vsClass.DEBUG = false;

vsClass.vsInitialized = false;

-- Register as event listener
addModEventListener(vsClass);

local vsClass_mt = Class(vsClass, Mission00);


function vsClass:new(baseDirectory, customMt)
    local mt = customMt;
    if mt == nil then
        mt = SampleModMap_mt;
    end;
    local self = SampleModMap:superClass():new(baseDirectory, mt);

    return self;
end;

function vsClass:loadSteerable(xmlFile)
  -- Try to find the name of the vehicle.
  -- Idea for this comes from FastSwitcher-Mod
  if self.name == nil then self.name = getXMLString(xmlFile, "vehicle.name.".. g_languageShort); end
  if self.name == nil then self.name = getXMLString(xmlFile, "vehicle.name"); end
  if self.name == nil then self.name = getXMLString(xmlFile, "vehicle#type"); end
  if self.name == nil then self.name = "Steerable"; end
end
Steerable.load  = Utils.appendedFunction(Steerable.load, vsClass.loadSteerable);

function vsClass:loadAttachable(xmlFile)
  -- Try to find the name of the attachable
  -- Idea for this comes from FastSwitcher-Mod
  if self.name == nil then self.name = getXMLString(xmlFile, "vehicle.name.".. g_languageShort); end
  if self.name == nil then self.name = getXMLString(xmlFile, "vehicle.name"); end
  if self.name == nil then self.name = getXMLString(xmlFile, "vehicle#type"); end
  if self.name == nil then self.name = "Attachable"; end
end
Attachable.load = Utils.appendedFunction(Attachable.load, vsClass.loadAttachable);


function vsClass:keyEvent(unicode, sym, modifier, isDown)
	if isDown then
	-- dprint("Key Down - Unicode: " .. tostring(unicode).. " sym: " .. tostring(sym) .. " modifier: " .. tostring(modifier) .. " isDown: " .. tostring(isDown));
		--[[
		if vsClass.showKeyPress then
			vsClass.lastKey.sym = sym;
			vsClass.lastKey.unicode = unicode;
			vsClass.lastKey.modifier = modifier;
			for i,j in pairs(InputBinding) do
				if j ~= nil then 
					if InputBinding.isPressed(j) then
						vsClass.lastKey.inBind = "InputBinding."..tostring(i);
					end
				end
			end
		end
		--]]
		
		if sym == 27 then -- Esc-Key
			vsClass.showSteerables = false;
			vsClass:saveVehicleOrder();
			vsClass.selectedLock = false;
		end
	end
end

function vsClass:moveUp()
	local oldIndex = vsClass.selectedIndex;
	vsClass.selectedIndex = vsClass.selectedIndex - 1;
	if vsClass.selectedIndex < 1 then
		vsClass.selectedIndex = table.getn(g_currentMission.steerables);
	end
	if vsClass.selectedLock then
		vsClass:reSort(oldIndex, vsClass.selectedIndex);
	end
end	

function vsClass:moveDown()
	local oldIndex = vsClass.selectedIndex;
	vsClass.selectedIndex = vsClass.selectedIndex + 1;
	if vsClass.selectedIndex > table.getn(g_currentMission.steerables) then 
		vsClass.selectedIndex = 1; 
	end
	if vsClass.selectedLock then
		vsClass:reSort(oldIndex, vsClass.selectedIndex);
	end
end

function vsClass:reSort(oldPos, newPos)
	local tempElement = g_currentMission.steerables[oldPos];
	local tempIndex = vsClass.vUserOrder[oldPos];
	table.remove(g_currentMission.steerables, oldPos);
	table.remove(vsClass.vUserOrder, oldPos);
	table.insert(g_currentMission.steerables, newPos, tempElement);
	table.insert(vsClass.vUserOrder, newPos, tempIndex);
	tempElement = nil;
end

function vsClass:toggleParkState(index)
	if not g_currentMission.steerables[index].vs_isParked then
		if not g_currentMission.steerables[index].isControlled then
			g_currentMission.steerables[index].vs_isParked = true;
			g_currentMission.steerables[index].isControlled = true;
			g_currentMission.steerables[index].controllerName = "<P>";
			g_currentMission.steerables[index]:setCharacterVisibility(false);
		end
	else
		g_currentMission.steerables[index].vs_isParked = false;
		g_currentMission.steerables[index].isControlled = false;
		g_currentMission.steerables[index].controllerName = "";
	end
end

function vsClass:update(dt)
	if not vsClass.vsInitialized  then  -- Improvised init since no proper API-Documentation available yet
		vsClass:vsInit();
		vsClass.vsInitialized = true;
	end
	
	if InputBinding.hasEvent(InputBinding.vs_toggleList) then
		if vsClass.showSteerables then
			vsClass.showSteerables = false;
			vsClass.selectedLock = false;
			vsClass:saveVehicleOrder();
		else
			vsClass.showSteerables = true;
		end
	end
	
	if vsClass.showSteerables then
		if InputBinding.hasEvent(InputBinding.vs_moveCursorUp) then
			vsClass:moveUp();
		elseif InputBinding.hasEvent(InputBinding.vs_moveCursorDown) then
			vsClass:moveDown();
		elseif InputBinding.hasEvent(InputBinding.vs_lockListItem) then 
			if not vsClass.selectedLock and vsClass.selectedIndex > 0 then
				vsClass.selectedLock = true;
			elseif vsClass.selectedLock then
				vsClass.selectedLock = false;
			end
		elseif InputBinding.hasEvent(InputBinding.vs_changeVehicle) then
			if g_currentMission.steerables[vsClass.selectedIndex].vs_isParked then
				vsClass:toggleParkState(vsClass.selectedIndex);
			end
			if g_currentMission.steerables[vsClass.selectedIndex].isControlled == false then
				g_client:getServerConnection():sendEvent(VehicleEnterRequestEvent:new(g_currentMission.steerables[vsClass.selectedIndex], g_settingsNickname));
			end;
		elseif InputBinding.hasEvent(InputBinding.vs_togglePark) then
			if g_currentMission.steerables[vsClass.selectedIndex].controllerName == g_settingsNickname then
				local newIndex = 0;
				for i = vsClass.selectedIndex, table.getn(g_currentMission.steerables) do
					if not g_currentMission.steerables[i].isControlled then
						newIndex = i;
						break;
					end
				end
				if newIndex == 0 then 
					for i =1,  vsClass.selectedIndex do
						if not g_currentMission.steerables[i].isControlled then
							newIndex = i;
							break;
						end
					end
				end
				if newIndex > 0 then
					g_client:getServerConnection():sendEvent(VehicleEnterRequestEvent:new(g_currentMission.steerables[newIndex], g_settingsNickname));
					vsClass:toggleParkState(vsClass.selectedIndex);
				else
					print("VehicleSort: Can't park vehicle - all other vehicles parked or occupied!");
				end
				
			else 
				vsClass:toggleParkState(vsClass.selectedIndex);
			end
		end
	end
end;


function vsClass:draw()
	local sCount = table.getn(g_currentMission.steerables);
	
	if vsClass.showSteerables and sCount > 0 then
		local lineCount = 0;
		local xPos = vsClass.tPos.x;
		local yPos = vsClass.tPos.y;
		setTextColor(unpack(vsClass.tColor.standard));
		setTextAlignment(vsClass.tPos.alignment);
		renderText(xPos, yPos + vsClass.tPos.size + vsClass.tPos.spacing + 0.007, vsClass.tPos.size + 0.005, g_i18n:getText("headline"));
		for i=1, sCount do
			if i == vsClass.selectedIndex then
				if vsClass.selectedLock then
					setTextColor(unpack(vsClass.tColor.locked));
				else
					setTextColor(unpack(vsClass.tColor.selected));
				end
			elseif g_currentMission.steerables[i].vs_isParked then
				setTextColor(unpack(vsClass.tColor.parked));
			else
				setTextColor(unpack(vsClass.tColor.standard));
			end
			if g_currentMission.steerables[i].isControlled and g_currentMission.steerables[i].controllerName == g_settingsNickname then
				setTextBold(true);
			else
				setTextBold(false);
			end
			renderText(xPos, yPos, vsClass.tPos.size, vsClass:getFullVehicleName(i));
			
			yPos = yPos - vsClass.tPos.size - vsClass.tPos.spacing;
			if yPos < vsClass.borderToTop then
				yPos = vsClass.tPos.y;
				xPos = xPos + vsClass.tPos.columnWidth;
			end
			
		end
	end
end

function vsClass:loadMap(name)
	vsClass.vsInitialized = false;
end;

function vsClass:deleteMap()
	vsClass.vInitOrder = {};
	vsClass.vUserOrder = {};
	vsClass.vsInitialized = false;
end;

function vsClass:mouseEvent(posX, posY, isDown, isUp, button)
end;

function vsClass:vsInit()
	vsClass.userPath = getUserProfileAppPath();
	vsClass.vsBasePath = vsClass.userPath .. "vehicleSort/";
	vsClass.vsSavePath = vsClass.vsBasePath .. "savegame" .. g_careerScreen.selectedIndex .. "/";
	createFolder(vsClass.vsBasePath);
	createFolder(vsClass.vsSavePath);
	
	vsClass.vsXML = vsClass.vsSavePath.."v_order.xml";
	
	vsClass:loadVehicleOrder();
	
	print("VehicleSort initialized");
end

function vsClass.addVehic(a, b)
	if b.isSteerable then 
		local listItem = {};
		listItem.ranking = #vsClass.vInitOrder + 1;
		listItem.id = b.id;
		table.insert(vsClass.vInitOrder, listItem);
		if vsClass.vsInitialized then
			local listItem = {};
			listItem.ranking = #vsClass.vUserOrder + 1;
			listItem.id = b.id;
			table.insert(vsClass.vUserOrder, listItem);
		end
		b.vs_isParked = false;
	end
end

function vsClass.removeVehic(a, b)
	if b.isSteerable then
		-- print("Steerable sold!");
		b.vs_isParked = nil;
		local remIndex = 0;
		local modRank = #vsClass.vUserOrder;
		for i,j in ipairs(vsClass.vUserOrder) do
			if j.id == b.id then
				remIndex = i;
				modRank = j.ranking
			end
		end
		if remIndex > 0 then
			table.remove(vsClass.vUserOrder, remIndex);
		else
			print("VehicleSort: Error removing sold Vehicle from User-List!");
		end
		for i,j in ipairs(vsClass.vUserOrder) do
			if j.ranking > modRank then
				j.ranking = j.ranking - 1;
			end
		end
	end
end

function vsClass:getFullVehicleName(index)
	local vehName = "";
	if index <= table.getn(g_currentMission.steerables) then
		if g_currentMission.steerables[index].vs_isParked then
			vehName = "<P> ";
		elseif g_currentMission.steerables[index].controllerName ~= g_settingsNickname and g_currentMission.steerables[index].controllerName ~= "Unknown" and g_currentMission.steerables[index].controllerName ~= "" and g_currentMission.steerables[index].controllerName ~= nil then
			vehName = "("..g_currentMission.steerables[index].controllerName..") ";
		end
		vehName = vehName..g_currentMission.steerables[index].name;
		if (g_currentMission.steerables[index].attachedImplements[1] ~= nil and g_currentMission.steerables[index].attachedImplements[1].object ~= nil) then
          vehName = vehName .. " - ".. g_i18n:getText("with") .." '".. Utils.getNoNil(g_currentMission.steerables[index].attachedImplements[1].object.name, "(implement1)") .."'";
          if (g_currentMission.steerables[index].attachedImplements[2] ~= nil and g_currentMission.steerables[index].attachedImplements[2].object ~= nil) then
            vehName = vehName .. " & '".. Utils.getNoNil(g_currentMission.steerables[index].attachedImplements[2].object.name, "(implement2)") .."'";
          end;
        end;
		return vehName;
	else
		return nil;
	end
end

function vsClass:saveVehicleOrder()
	vsClass.saveFile = createXMLFile("vsClass.saveFile", vsClass.vsXML, "vOrder");
	dprint(#vsClass.vUserOrder.." needs to be saved!");
	for i=1, #vsClass.vUserOrder do
		dprint("Saving vehicle "..i);
		setXMLInt(vsClass.saveFile, "vOrder.vehicle"..i.."#id", vsClass.vUserOrder[i].ranking);
		setXMLBool(vsClass.saveFile, "vOrder.vehicle"..i.."#parked", g_currentMission.steerables[i].vs_isParked);
	end
	saveXMLFile(vsClass.saveFile);
end

function vsClass:loadVehicleOrder()
	vsClass.saveFile = createXMLFile("vsClass.loadFile", vsClass.vsXML, "vOrder");
	vsClass.saveFile = loadXMLFile("vsClass.loadFile", vsClass.vsXML);
	if not hasXMLProperty(vsClass.saveFile, "vOrder") then
		print("VehicleSort: No Save-File found - using standard vehicle order!");
		for i = 1, table.getn(g_currentMission.steerables) do
			local listItem = {};
			listItem.ranking = i;
			listItem.id = g_currentMission.steerables[i].id;
			vsClass.vUserOrder[i]= listItem;
		end
	else
		local vIndex = 1;
		local savedIDs = {};
		local savedParkStates = {};
		while hasXMLProperty(vsClass.saveFile, "vOrder.vehicle"..vIndex.."#id") do
			table.insert(savedIDs, getXMLInt(vsClass.saveFile, "vOrder.vehicle"..vIndex.."#id"));
			table.insert(savedParkStates, getXMLBool(vsClass.saveFile, "vOrder.vehicle"..vIndex.."#parked"));
			dprint("Recognized Order Index: "..savedIDs[vIndex]);
			vIndex = vIndex + 1;
		end
		local vTSize = table.getn(g_currentMission.steerables);
		if #savedIDs == vTSize then
			dprint("Size Match savedIDs ("..#savedIDs..") <-> globalVehicleTableSize ("..vTSize..")!");
			for i = 1, vTSize do
				dprint("Moving away from Table Position "..i);
				local listItem = {};
				listItem.ranking = tonumber(savedIDs[i]);
				vsClass.vUserOrder[i] = listItem;
				local eBreaker = 0;
				while vsClass.vInitOrder[i].ranking ~= vsClass.vUserOrder[i].ranking and eBreaker <= (vTSize + 10) do
					dprint("Moving ID "..vsClass.vInitOrder[i].ranking.. " - eBreaker: "..eBreaker);
					eBreaker = eBreaker + 1;
					local tempIndex = vsClass.vInitOrder[i];
					local tempVehicle = g_currentMission.steerables[i];
					
					table.remove(vsClass.vInitOrder, i);
					table.remove(g_currentMission.steerables, i);
					
					table.insert(vsClass.vInitOrder, vTSize, tempIndex);
					table.insert(g_currentMission.steerables, vTSize, tempVehicle);
				end
			end
			for i=1, vTSize do
				vsClass.vUserOrder[i].id = g_currentMission.steerables[i].id;
			end
			for index, isParked in ipairs(savedParkStates) do
				if isParked then 
					vsClass:toggleParkState(index);
				end
			end
		else
			print("VehicleSort: SaveFile corrupt! Using standard vehicle order!");
			for i = 1, table.getn(g_currentMission.steerables) do
				local listItem = {};
				listItem.ranking = i;
				listItem.id = g_currentMission.steerables[i].id;
				vsClass.vUserOrder[i] = listItem;
			end
		end
	end
end

function testFunc()
	
end

function dprint(aString)
	if vsClass.DEBUG then print("******* VEHICLE SORT DEBUG: *******:    "..tostring(aString)); end
end

vsClass:superClass():superClass().addVehicle = Utils.appendedFunction(vsClass:superClass():superClass().addVehicle, vsClass.addVehic);
vsClass:superClass():superClass().removeVehicle = Utils.appendedFunction(vsClass:superClass():superClass().removeVehicle, vsClass.removeVehic);

print("VehicleSort loaded");


